//+------------------------------------------------------------------+
//|                                               Orthogonal CCI.mq5 |
//|                                                        AIS Forex |
//|                        https://www.mql5.com/ru/users/aleksej1966 |
//+------------------------------------------------------------------+
#property copyright "AIS Forex"
#property link      "https://www.mql5.com/ru/users/aleksej1966"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_plots   1

#property indicator_type1  DRAW_LINE
#property indicator_label1 "Orth. CCI"
#property indicator_color1 clrBlue
#property indicator_width1 1
#property indicator_style1 STYLE_SOLID

enum type {Legendre,ChebyshevI,ChebyshevII,Laguerre,Hermite,ChebHermite};

input type Type=Legendre;
input ushort iPeriod=15,
             N=1;

struct polynom {double c[];double denom;};
polynom array[];
int period,n;
double buffer[],weight[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,buffer,INDICATOR_DATA);
   ArraySetAsSeries(buffer,true);

   n=MathMin(9,N);
   period=MathMax(n+1,iPeriod);

   double x[];
   ArrayResize(x,period);
   for(int i=0;i<period;i++)
      x[i]=Type==Laguerre? period-i-1:(period-2*i-1.)/(period-1);

   if(Type==Hermite || Type==ChebHermite)
      for(int i=0;i<period;i++)
         x[i]=(period-2*i-1.)/2;

   ArrayResize(array,n);
   ArrayResize(weight,n);

   ArrayResize(array[0].c,period);
   ArrayInitialize(array[0].c,1.);
   array[0].denom=period;

   for(int i=1;i<n;i++)
     {
      ArrayResize(array[i].c,period);
      ArrayInitialize(array[i].c,0);
      array[i].denom=0;
      double d=0;
      for(int j=0;j<period;j++)
        {
         array[i].c[j]=CalcCoeff(i,x[j]);
         d=d+array[i].c[j];
        }
      d=d/period;
      for(int j=0;j<period;j++)
        {
         array[i].c[j]=array[i].c[j]-d;
         array[i].denom=array[i].denom+MathPow(array[i].c[j],2);
        }
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   if(rates_total>prev_calculated)
     {
      ArraySetAsSeries(open,true);

      int bars=prev_calculated>0? rates_total-prev_calculated-1:rates_total-period-1;

      for(int b=bars;b>=0;b--)
        {
         for(int i=0;i<n;i++)
           {
            weight[i]=0;
            for(int j=0;j<period;j++)
               weight[i]=weight[i]+array[i].c[j]*open[j+b];
            weight[i]=weight[i]/array[i].denom;
           }

         double dev=0,dif=0;
         for(int i=0;i<period;i++)
           {
            double res=0;
            for(int j=0;j<n;j++)
               res=res+weight[j]*array[j].c[i];

            if(i==0)
               dif=open[b+i]-res;

            dev=dev+MathAbs(open[b+i]-res);
           }

         buffer[b]=200*dif*period/(3*dev);
        }
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CalcCoeff(int lvl,double x)
  {
//---
   double res=0;

   if(lvl==1)
     {
      res=x;
      if(Type==ChebyshevII || Type==ChebHermite)
         res=2*x;
      if(Type==Laguerre)
         res=-x+1;
     }

   if(lvl==2)
     {
      if(Type==Legendre)
         res=(3*MathPow(x,2)-1)/2;
      if(Type==ChebyshevI)
         res=2*MathPow(x,2)-1;
      if(Type==ChebyshevII)
         res=4*MathPow(x,2)-1;
      if(Type==Laguerre)
         res=(MathPow(x,2)-4*x+2)/2;
      if(Type==Hermite)
         res=MathPow(x,2)-1;
      if(Type==ChebHermite)
         res=4*MathPow(x,2)-2;
     }

   if(lvl==3)
     {
      if(Type==Legendre)
         res=(5*MathPow(x,3)-3*x)/2;
      if(Type==ChebyshevI)
         res=4*MathPow(x,3)-3*x;
      if(Type==ChebyshevII)
         res=8*MathPow(x,3)-4*x;
      if(Type==Laguerre)
         res=(-MathPow(x,3)+9*MathPow(x,2)-18*x+6)/6;
      if(Type==Hermite)
         res=MathPow(x,3)-3*x;
      if(Type==ChebHermite)
         res=8*MathPow(x,3)-12*x;
     }

   if(lvl==4)
     {
      if(Type==Legendre)
         res=(35*MathPow(x,4)-30*MathPow(x,2)+3)/8;
      if(Type==ChebyshevI)
         res=8*MathPow(x,4)-8*MathPow(x,2)+1;
      if(Type==ChebyshevII)
         res=16*MathPow(x,4)-12*MathPow(x,2)+1;
      if(Type==Laguerre)
         res=(MathPow(x,4)-16*MathPow(x,3)+72*MathPow(x,2)-96*x+24)/24;
      if(Type==Hermite)
         res=MathPow(x,4)-6*MathPow(x,2)+3;
      if(Type==ChebHermite)
         res=16*MathPow(x,4)-48*MathPow(x,2)+12;
     }

   if(lvl==5)
     {
      if(Type==Legendre)
         res=(63*MathPow(x,5)-70*MathPow(x,3)+15*x)/8;
      if(Type==ChebyshevI)
         res=16*MathPow(x,5)-20*MathPow(x,3)+5*x;
      if(Type==ChebyshevII)
         res=32*MathPow(x,5)-32*MathPow(x,3)+6*x;
      if(Type==Laguerre)
         res=(-MathPow(x,5)+25*MathPow(x,4)-200*MathPow(x,3)+600*MathPow(x,2)-600*x+120)/120;
      if(Type==Hermite)
         res=MathPow(x,5)-10*MathPow(x,3)+15*x;
      if(Type==ChebHermite)
         res=32*MathPow(x,5)-160*MathPow(x,3)+120*x;
     }

   if(lvl==6)
     {
      if(Type==Legendre)
         res=(231*MathPow(x,6)-315*MathPow(x,4)+105*MathPow(x,2)-5)/16;
      if(Type==ChebyshevI)
         res=32*MathPow(x,6)-48*MathPow(x,4)+18*MathPow(x,2)-1;
      if(Type==ChebyshevII)
         res=64*MathPow(x,6)-80*MathPow(x,4)+24*MathPow(x,2)-1;
      if(Type==Laguerre)
         res=(MathPow(x,6)-36*MathPow(x,5)+450*MathPow(x,4)-2400*MathPow(x,3)+5400*MathPow(x,2)-4320*x+720)/720;
      if(Type==Hermite)
         res=MathPow(x,6)-15*MathPow(x,4)+45*MathPow(x,2)-15;
      if(Type==ChebHermite)
         res=64*MathPow(x,6)-480*MathPow(x,4)+720*MathPow(x,2)-120;
     }

   if(lvl==7)
     {
      if(Type==Legendre)
         res=(429*MathPow(x,7)-693*MathPow(x,5)+315*MathPow(x,3)-35*x)/16;
      if(Type==ChebyshevI)
         res=64*MathPow(x,7)-112*MathPow(x,5)+56*MathPow(x,3)-7*x;
      if(Type==ChebyshevII)
         res=128*MathPow(x,7)-192*MathPow(x,5)+80*MathPow(x,3)-8*x;
      if(Type==Laguerre)
         res=(-MathPow(x,7)+49*MathPow(x,6)-882*MathPow(x,5)+7350*MathPow(x,4)-29400*MathPow(x,3)+52920*MathPow(x,2)-35280*x+5040)/5040;
      if(Type==Hermite)
         res=MathPow(x,7)-21*MathPow(x,5)+105*MathPow(x,3)-105*x;
      if(Type==ChebHermite)
         res=128*MathPow(x,7)-1344*MathPow(x,5)+3360*MathPow(x,3)-1680*x;
     }

   if(lvl==8)
     {
      if(Type==Legendre)
         res=(6435*MathPow(x,8)-12012*MathPow(x,6)+6930*MathPow(x,4)-1260*MathPow(x,2)+35)/128;
      if(Type==ChebyshevI)
         res=128*MathPow(x,8)-256*MathPow(x,6)+160*MathPow(x,4)-32*MathPow(x,2)+1;
      if(Type==ChebyshevII)
         res=256*MathPow(x,8)-448*MathPow(x,6)+240*MathPow(x,4)-40*MathPow(x,2)+1;
      if(Type==Laguerre)
         res=(MathPow(x,8)-64*MathPow(x,7)+1568*MathPow(x,6)-18816*MathPow(x,5)+117600*MathPow(x,4)-376320*MathPow(x,3)+564480*MathPow(x,2)-322560*x+40320)/40320;
      if(Type==Hermite)
         res=MathPow(x,8)-28*MathPow(x,6)+210*MathPow(x,4)-420*MathPow(x,2)+105;
      if(Type==ChebHermite)
         res=256*MathPow(x,8)-3584*MathPow(x,6)+13440*MathPow(x,4)-13440*MathPow(x,2)+1680;
     }

   if(lvl==9)
     {
      if(Type==Legendre)
         res=(12155*MathPow(x,9)-25740*MathPow(x,7)+18018*MathPow(x,5)-4620*MathPow(x,3)+315*x)/128;
      if(Type==ChebyshevI)
         res=256*MathPow(x,9)-576*MathPow(x,7)+432*MathPow(x,5)-120*MathPow(x,3)+9*x;
      if(Type==ChebyshevII)
         res=512*MathPow(x,9)-1024*MathPow(x,7)+672*MathPow(x,5)-160*MathPow(x,3)+10*x;
      if(Type==Laguerre)
         res=(-MathPow(x,9)+81*MathPow(x,8)-2592*MathPow(x,7)+42336*MathPow(x,6)-381024*MathPow(x,5)+1905120*MathPow(x,4)-5080320*MathPow(x,3)+6531840*MathPow(x,2)-3265920*x+362880)/362880;
      if(Type==Hermite)
         res=MathPow(x,9)-36*MathPow(x,7)+378*MathPow(x,5)-1260*MathPow(x,3)+945*x;
      if(Type==ChebHermite)
         res=512*MathPow(x,9)-9216*MathPow(x,7)+48384*MathPow(x,5)-80640*MathPow(x,3)+30240*x;
     }

   return(res);
//---
  }
//+------------------------------------------------------------------+
